#ifndef T_Conf_channelSAN_H_
#define T_Conf_channelSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern double v_delta;
extern double t_delta;
extern double D_T_attack_prob;
extern double T_V_Offset_mean;
extern double T_V_Offset_stddev;
extern double T_V_step_interval_mean;
extern short T_V_steps_min;
extern short T_V_steps_max;
extern double t_window;
extern double torque_req_rate;
extern UserDistributions* TheDistribution;

void MemoryError();


/*********************************************************************
               T_Conf_channelSAN Submodel Definition                   
*********************************************************************/

class T_Conf_channelSAN:public SANModel{
public:

class torque_loop_stepActivity:public Activity {
public:

  Place* Attack_in_progress;
  short* Attack_in_progress_Mobius_Mark;
  Place* T_increment;
  short* T_increment_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  ExtendedPlace<double>* Detection_prob;
  ExtendedPlace<double>* T_value_step;
  ExtendedPlace<double>* step_rate;
  ExtendedPlace<double>* T_a_cur_value;
  ExtendedPlace<double>* T_cur_value;
  ExtendedPlace<double>* Last_Valid_Torque_timestamp;

  double* TheDistributionParameters;
  torque_loop_stepActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // torque_loop_stepActivityActivity

class Torque_requestActivity:public Activity {
public:

  Place* Torque_state;
  short* Torque_state_Mobius_Mark;
  Place* T_increment;
  short* T_increment_Mobius_Mark;
  ExtendedPlace<double>* T_cur_value;
  ExtendedPlace<double>* Last_Valid_Torque_timestamp;
  Place* FalseAlarmFlag;
  short* FalseAlarmFlag_Mobius_Mark;
  Place* CH1_SS_OK;
  short* CH1_SS_OK_Mobius_Mark;
  Place* CH2_SS_OK;
  short* CH2_SS_OK_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* System_safe;
  short* System_safe_Mobius_Mark;
  Place* System_unsafe;
  short* System_unsafe_Mobius_Mark;
  Place* Attack_Active;
  short* Attack_Active_Mobius_Mark;
  Place* Detected;
  short* Detected_Mobius_Mark;
  Place* Missed_OK;
  short* Missed_OK_Mobius_Mark;
  Place* Missed_unsafe;
  short* Missed_unsafe_Mobius_Mark;

  double* TheDistributionParameters;
  Torque_requestActivity();
  ~Torque_requestActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // Torque_requestActivityActivity

class Detection_windowActivity:public Activity {
public:

  Place* Attack_Active;
  short* Attack_Active_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* Detected;
  short* Detected_Mobius_Mark;
  Place* CH1_SS_OK;
  short* CH1_SS_OK_Mobius_Mark;
  Place* CH2_SS_OK;
  short* CH2_SS_OK_Mobius_Mark;
  Place* System_safe;
  short* System_safe_Mobius_Mark;
  Place* System_unsafe;
  short* System_unsafe_Mobius_Mark;
  Place* T_increment;
  short* T_increment_Mobius_Mark;
  Place* Missed_unsafe;
  short* Missed_unsafe_Mobius_Mark;
  Place* Missed_OK;
  short* Missed_OK_Mobius_Mark;

  double* TheDistributionParameters;
  Detection_windowActivity();
  ~Detection_windowActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // Detection_windowActivityActivity

class T_inc_stepActivity:public Activity {
public:

  Place* T_increment;
  short* T_increment_Mobius_Mark;
  Place* Attack_Active;
  short* Attack_Active_Mobius_Mark;
  ExtendedPlace<double>* T_cur_value;
  ExtendedPlace<double>* T_a_cur_value;
  ExtendedPlace<double>* T_value_step;
  ExtendedPlace<double>* Last_Valid_Torque_timestamp;
  ExtendedPlace<short>* Debugging;
  Place* Detected;
  short* Detected_Mobius_Mark;
  Place* Missed_unsafe;
  short* Missed_unsafe_Mobius_Mark;
  Place* Missed_OK;
  short* Missed_OK_Mobius_Mark;
  ExtendedPlace<double>* step_rate;

  double* TheDistributionParameters;
  T_inc_stepActivity();
  ~T_inc_stepActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // T_inc_stepActivityActivity

  //List of user-specified place names
  Place* Attack_in_progress;
  Place* T_increment;
  Place* Torque_state;
  Place* Detected;
  Place* System_unsafe;
  Place* System_safe;
  Place* System_OK;
  Place* Attack_Active;
  Place* Missed_unsafe;
  Place* Missed_OK;
  Place* CH1_SS_OK;
  Place* CH2_SS_OK;
  Place* FalseAlarmFlag;
  ExtendedPlace<double>* Detection_prob;
  ExtendedPlace<double>* Last_Valid_Torque_timestamp;
  ExtendedPlace<double>* T_value_step;
  ExtendedPlace<double>* T_cur_value;
  ExtendedPlace<double>* step_rate;
  ExtendedPlace<double>* T_a_cur_value;
  ExtendedPlace<short>* Debugging;

  // Create instances of all actvities
  torque_loop_stepActivity torque_loop_step;
  Torque_requestActivity Torque_request;
  Detection_windowActivity Detection_window;
  T_inc_stepActivity T_inc_step;
  //Create instances of all groups 
  PreselectGroup ImmediateGroup;
  PostselectGroup torque_loop_stepGroup;

  T_Conf_channelSAN();
  ~T_Conf_channelSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end T_Conf_channelSAN

#endif // T_Conf_channelSAN_H_
